package models

import (
	"context"
	"time"

	"github.com/beego/beego/v2/client/orm"
)

type User struct {
	UserId         int       `orm:"pk" description:"用户id"`
	RealName       string    `description:"姓名"`
	Phone          string    `description:"手机号"`
	AreaCode       string    `description:"区号"`
	Email          string    `description:"邮箱"`
	SellerId       int       `description:"销售id（SysUserId）"`
	Company        string    `description:"所属公司"`
	ValidStartTime time.Time `description:"有效期开始时间"`
	ValidEndTime   time.Time `description:"有效期结束时间"`
	Status         int       `description:"用户类型: 0表示禁用，1表示潜在客户，2表示正式客户"`
	CreateTime     time.Time `description:"系统中首次新增用户的时间"`
	ModifyTime     time.Time `description:"系统中用户信息变更的时间"`
	RegisterTime   time.Time `description:"用户首次登录小程序的时间"`
	IsSubscribed   bool      `description:"是否关注公众号: 0表示没有关注，1表示关注"`
	IsRegistered   bool      `description:"是否注册: 0表示没有注册，1表示注册"`
}

func (u *User) Save() (err error) {
	o := orm.NewOrmUsingDB("master")
	_, err = o.InsertOrUpdate(u)
	return
}

func (u *User) Update(cols []string) (err error) {
	o := orm.NewOrmUsingDB("master")
	_, err = o.Update(u, cols...)
	return
}

func UpdateUserStatus(condition string, pars []interface{}) (err error) {
	o := orm.NewOrmUsingDB("master")
	sql := ` UPDATE user SET status=0 WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	_, err = o.Raw(sql, pars).Exec()
	return
}

func SaveUser(user *User, chartPermissionIds []int) (err error) {
	o := orm.NewOrmUsingDB("master")
	err = o.DoTx(func(ctx context.Context, txOrm orm.TxOrmer) error {
		insertId, er := txOrm.InsertOrUpdate(user)
		if er != nil {
			return er
		}
		if user.UserId != 0 {
			insertId = int64(user.UserId)
		}
		// 先删除再增加
		sql := `DELETE FROM user_chart_permission_mapping WHERE user_id=?`
		_, er = txOrm.Raw(sql, insertId).Exec()
		if er != nil {
			return er
		}
		for _, id := range chartPermissionIds {
			userChartPermissionMapping := new(UserChartPermissionMapping)
			userChartPermissionMapping.UserId = int(insertId)
			userChartPermissionMapping.ChartPermissionId = id
			_, er = txOrm.Insert(userChartPermissionMapping)
			if er != nil {
				return er

			}
		}
		return nil
	})
	return
}

func GetUserByPhone(phone, areaCode string) (item *User, err error) {
	o := orm.NewOrmUsingDB("master")
	sql := `SELECT * FROM user WHERE phone=? AND area_code=?`
	err = o.Raw(sql, phone, areaCode).QueryRow(&item)
	return
}

func GetUserByEmail(email string) (item *User, err error) {
	o := orm.NewOrmUsingDB("master")
	sql := `SELECT * FROM user WHERE email=? `
	err = o.Raw(sql, email).QueryRow(&item)
	return
}

func GetUserById(userId int) (item *User, err error) {
	o := orm.NewOrmUsingDB("master")
	sql := `SELECT * FROM user WHERE user_id=? `
	err = o.Raw(sql, userId).QueryRow(&item)
	return
}

func GetUserList(condition string, pars []interface{}, startSize, pageSize int) (items []*User, err error) {
	sql := `SELECT * FROM user WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	sql += ` ORDER BY modify_time DESC LIMIT ?,? `
	o := orm.NewOrmUsingDB("master")
	_, err = o.Raw(sql, pars, startSize, pageSize).QueryRows(&items)
	return
}

func GetUserCount(condition string, pars []interface{}) (count int, err error) {
	sql := `SELECT COUNT(*) AS count FROM user WHERE 1=1 `
	if condition != "" {
		sql += condition
	}
	o := orm.NewOrmUsingDB("master")
	err = o.Raw(sql, pars...).QueryRow(&count)
	return
}

func DeleteUserById(userId int) (err error) {
	o := orm.NewOrmUsingDB("master")
	err = o.DoTx(func(ctx context.Context, txOrm orm.TxOrmer) error {
		sql := `DELETE FROM user WHERE user_id=?`
		_, e := txOrm.Raw(sql, userId).Exec()
		if e != nil {
			return e
		}
		sql = `DELETE FROM user_chart_permission_mapping WHERE user_id=?`
		_, e = txOrm.Raw(sql, userId).Exec()
		if e != nil {
			return e
		}
		return nil
	})
	return
}
